@@ -1,11 +1,17 @@
-# Twitter base endpoint for direct api calls
-t_endpoint <- "https://api.twitter.com/1.1/"
+# Twitter base endpoint for direct API calls
+t_endpoint <- list(
+  `1.1` = "https://api.twitter.com/1.1/",
+  `2`  = "https://api.twitter.com/2/"
+)
 
 # Twitter endpoint for getting rate limit status
-ratelimit_endpoint <- paste(t_endpoint, "application/rate_limit_status.json", sep = "")
+ratelimit_endpoint <- paste(t_endpoint[["1.1"]], "application/rate_limit_status.json", sep = "")
 
 # Twitter endpoint for searching tweets
-search_endopoint <-  paste(t_endpoint, "search/tweets.json", sep = "")
+search_endpoint <-  list(
+  `1.1` = paste(t_endpoint[["1.1"]], "search/tweets.json", sep = ""),
+  `2` =  paste(t_endpoint[["2"]], "tweets/search/recent", sep = "")
+)
 
 # Get twitter token bases on configuration based on configuration settings (app or user)
 # This function is called when saving properties on shiny app and first time epitweetr performs a twitter search in a session
@@ -39,15 +45,28 @@ get_token <- function(request_new = TRUE) {
   } 
   return(token)
 }
-
+last_get <- new.env()
 # Execute a get request on twitter API using the provided URL
-# It wil internally deal with token generation and rate limits
-# If request fails with an error or warning it will retry with a cuadratic waiting time
-# Twitter response codes are interpreted as per twitter documention to slow down and wait as requested and to respect rate limits
+# It will internally deal with token generation and rate limits
+# If request fails with an error or warning it will retry with a quadratic waiting time
+# Twitter response codes are interpreted as per twitter documentation to slow down and wait as requested and to respect rate limits
 # url: URL to try
 # i: Number of tries done already
 # This files to file.path(conf$data_dir, "lasterror.log") the last encountered error produced if any
-twitter_get <- function(url, i = 0, retries = 20) {
+twitter_get <- function(urls, i = 0, retries = 20, tryed = list()) {
+  # Getting the right url to try based on version
+  url <- if(length(urls) > 1) {
+    toTry = order(names(urls)[!names(urls) %in% tryed], decreasing=T)
+    if(exists("api_ver", last_get))
+      toTry <- toTry[order(sapply(toTry, function(v) if(v == last_get$api_ver) paste0("zzz", v) else v), decreasing=T)]
+    toTry <- toTry[[1]]
+    last_get$api_ver = toTry
+    urls[[toTry]]
+  } else {
+    last_get$api_ver = names(urls)[[1]]
+    urls[[1]]
+  }
+  # message(url)
   # Getting token if not set for current session
   if(is.null(conf$token) || !exists("token", where = conf)) conf$token <- get_token()
   # Trying to get the response quadratic waiting time is implemented if the URL fails 
@@ -64,24 +83,30 @@ twitter_get <- function(url, i = 0, retries = 20) {
         towait = i * i
         save_config(data_dir = conf$data_dir, topics = TRUE, properties = FALSE)
         Sys.sleep(towait)  
-        return(twitter_get(url, i = i + 1, retries = retries))
+        return(twitter_get(urls, i = i + 1, retries = retries))
       }, error = function(error_condition) {
         message(paste("retrying because of error ", error_condition))
         towait = i * i
         save_config(data_dir = conf$data_dir, topics = TRUE, properties = FALSE)
         Sys.sleep(towait)  
-        return(twitter_get(url, i = i + 1, retries = retries))
+        return(twitter_get(urls, i = i + 1, retries = retries))
         
     })
   if(res$status_code == 200) {
     # Status code is 200 (OK) 
     if(exists("x-rate-limit-remaining", where = res$headers) && as.integer(res$headers[["x-rate-limit-remaining"]]) == 0 ) {
-      # Rate limit has been reacched
+      # Rate limit has been reached
       conf$token = NULL
-      if(exists("x-rate-limit-reset", where = res$headers)) {
-        # Sleeping the requsted number of secons 
+      if(length(tryed) < length(url)) {
+        # there is still another endpoint to test
+        message(paste("The Twitter application endpoint ",last_get$api_ver ,"is about to reach its rate limit, changing endpoint to another available version"))
+        tryed <- c(tryed, last_get$api_ver)
+        return(twitter_get(urls, i = i + 1, retries = retries, tryed = tryed))
+
+      } else if(exists("x-rate-limit-reset", where = res$headers)) {
+        # Sleeping the requested number of seconds 
         towait <- as.integer(floor(as.numeric(difftime(as.POSIXct(as.integer(res$headers[["x-rate-limit-reset"]]),  origin="1970-01-01"),Sys.time(), units = "secs"))))  
-        message(paste("The Twitter application endpoint is about to reach its rate limit, waiting", towait,"seconds  until", Sys.time() + towait, " as requested by Twitter"))
+        message(paste("The Twitter application endpoint is ",last_get$api_ver ," about to reach its rate limit, waiting", towait,"seconds  until", Sys.time() + towait, " as requested by Twitter"))
         if(towait < 1) {
           towait = i * i
         } else {
@@ -93,30 +118,38 @@ twitter_get <- function(url, i = 0, retries = 20) {
         # If no waiting time has been provided (should not happen) waiting 15 minutes which is the default rate limit window
         towait <- 15*60 
         save_config(data_dir = conf$data_dir, topics = TRUE, properties = FALSE)
-        message(paste("The Twitter application endpoint is about to reach its rate limit, but no waiting instruction has been detected. Waiting", towait,"seconds until", Sys.time()+towait))
+        message(paste("The Twitter application endpoint ",last_get$api_ver ,"is about to reach its rate limit, but no waiting instruction has been detected. Waiting", towait,"seconds until", Sys.time()+towait))
         Sys.sleep(towait)  
       }
     }
-    return(res)
+    return(httr::content(res,as="text"))
   }
   else if(res$status_code == 420 && i <= retries) {
     # Adding 60 seconds wait if application is being requested to slow down, and retries is less than limit
-    message("The Twitter application is being slown down (rate limited) by Twitter, waiting 1 minute before continuing")
+    message("The Twitter application is slowing down (rate limited) by Twitter, waiting 1 minute before continuing")
     save_config(data_dir = conf$data_dir, topics = TRUE, properties = FALSE)
     Sys.sleep(60)
     conf$token = NULL
-    return(twitter_get(url, i = i + 1, retries = retries))
+    return(twitter_get(urls, i = i + 1, retries = retries, tryed = tryed))
   } else if(res$status_code == 429 && i < retries) {
     # If application is rate limited, waiting and retrying
-    if(exists("x-rate-limit-reset", where = res$headers)) {
+    if(length(tryed) < length(url)) {
+      # there is still another endpoint to test
+      message(paste("The Twitter application endpoint ",last_get$api_ver ,"has reached its rate limit, changing endpoint to another available version"))
+      tryed <- c(tryed, last_get$api_ver)
+    } else if(exists("x-rate-limit-reset", where = res$headers)) {
       # Calculating the number of seconds requested to wait  
       towait <- as.integer(floor(as.numeric(difftime(as.POSIXct(as.integer(res$headers[["x-rate-limit-reset"]]),  origin="1970-01-01"),Sys.time(), units = "secs"))))  
-      message(paste("The Twitter application endpoint has reached its rate limit, waiting", towait,"seconds  until", Sys.time()+towait, "as requested by twitter"))
+      message(paste("The Twitter application endpoint ",last_get$api_ver ,"has reached its rate limit, waiting", towait,"seconds  until", Sys.time()+towait, "as requested by twitter"))
       if(towait < 1) {
         towait = i * i
       } else {
         towait = towait + 5  
       }
+      #resetting tries and last_get to start all over again
+      tryed <- list()
+      rm("api_ver", envir = last_get)
+
       save_config(data_dir = conf$data_dir, topics = TRUE, properties = FALSE)
       Sys.sleep(towait)  
     } else {
@@ -127,20 +160,33 @@ twitter_get <- function(url, i = 0, retries = 20) {
       Sys.sleep(towait + 10)  
     }
     conf$token = NULL
-    #Liogging the obtained error
+    # Logging the obtained error
     writeLines(httr::content(res,as="text"), paste(conf$data_dir, "lasterror.log", sep = "/"))
-    return(twitter_get(url, i = i + 1, retries = retries))
-  } else if(res$status_code <= 401 || res$status_code <= 403) {
+    return(twitter_get(urls, i = i + 1, retries = retries, tryed = tryed))
+  } else if(res$status_code >= 401 && res$status_code <= 403 && i >= retries) {
     # Stopping if non authorized by twitter
     writeLines(httr::content(res,as="text"), paste(conf$data_dir, "lasterror.log", sep = "/"))
-    stop(paste("Unauthorized by twitter API"))
+    message(res$status_code)
+    stop(paste("Unauthorized by twitter APIi", res$status_code))
+  } else if(last_get$api_ver == "2" && {
+      mess <- jsonlite::fromJSON(httr::content(res,as="text"))
+      if(is.na(is.na(mess$errors$parameters) || is.na(names(mess$errors$parameters))))
+        FALSE
+      else 
+        names(mess$errors$parameters) == "since_id"
+    }) {
+    # Special error in API 2 when query is too old
+    message("Error interpreted as too old query on Twitter API v2. Assuming empty result set")
+    message(mess)
+    stop("too-old")
   } else if(i <= retries) {
-      # Other non expected cases, retrying with a quadratic waiting rule at most (retries - i) more times
+      # Other non-expected cases, retrying with a quadratic waiting rule at most (retries - i) more times
       message(paste("Error status code ",res$status_code,"returned by Twitter API waiting for", i*i, "seconds before retry with a new token"))  
+      message(httr::content(res,as="text"))
       save_config(data_dir = conf$data_dir, topics = TRUE, properties = FALSE)
       Sys.sleep(i * i)
       conf$token = NULL
-      return(twitter_get(url, i = i + 1, retries = retries))
+      return(twitter_get(urls, i = i + 1, retries = retries, tryed = tryed))
   } else {
      #Run out of retries, stopping
      writeLines(httr::content(res,as="text"), paste(conf$data_dir, "lasterror.log", sep = "/"))
