@@ -1,8 +1,7 @@
-package demy.mllib.test
+package org.ecdc.epitweetr.test
 
 import org.scalatest._
 
-
 class UnitTest()  extends FlatSpec {
   def getSpark = SharedSpark.getSpark
 }
